<?php

if (!defined('BASEPATH'))
    exit('No direct script access allowed');

require_once(FCPATH . '/application/libraries/base_table.php');

class Tasks_table extends Base_table {

	function __construct()
	{
        parent::__construct(array('tbl'=>'project_task', 'tbl_key'=>'task_id'));
        $this->CI->load->model('tasks');
	}

	protected function get_closed_status_id()
	{
		$this->CI->db->select('task_status_id');
		$this->CI->db->where('entity_id', $this->entity_id);
		$this->CI->db->where('is_closed', 1);
		$this->CI->db->where('system_defined', 1);

		$query	= $this->CI->db->get('task_status');
		$result	= $query->row_array();
		if (!empty($result) && $result['task_status_id'])
		{
			return $result['task_status_id'];
		}

		return false;
	}

	private function setTaskStatus($project_id, $parent_task_id, $status_id)
	{
		// get parent task
		$this->CI->db->select('task_id, parent_task_id, is_closed');
		$this->CI->db->from('project_task');
		$this->CI->db->where(array(
			'entity_id'	=> $this->entity_id,
			'project_id'=> $project_id,
			'task_id'	=> $parent_task_id,
		));
		$query	= $this->CI->db->get();
		$parent_task	= $query->row_array();

		// Get unclosed childrens task list count
		$query	= $this->CI->db->select('count(task_id) AS cnt')->from('project_task')->where(array(
			'entity_id'	=> $this->entity_id,
			'project_id'=> $project_id,
			'parent_task_id'	=> $parent_task_id,
			'is_closed !='	=> '1'
		))->get();
		$child_count	= $query->result_array();

		// some childs are not closed
		if (!empty($child_count) && $child_count[0]['cnt'])
		{
			// open the parent.
			$this->CI->db->where(array(
				'entity_id'	=> $this->entity_id,
				'project_id'=> $project_id,
				'task_id'	=> $parent_task_id,
			))->update('project_task', array(
				'is_closed'		=> '0',
				'task_status_id'=> NULL
			));
		}
		else
		{
			// close the parent.
			$this->CI->db->where(array(
				'entity_id'	=> $this->entity_id,
				'project_id'=> $project_id,
				'task_id'	=> $parent_task_id,
			))->update('project_task', array(
				'is_closed'		=> '1',
				'task_status_id'=> $status_id
			));
		}

		// Apply same for parent of parent.
		if (!empty($parent_task['parent_task_id']))
		{
			$this->setTaskStatus($project_id, $parent_task['parent_task_id'], $status_id);
		}

		return true;
	}

// 	TC - Task Created.
// 	TE - Task detail updated/edited.
// 	TA - Task Assigned.
// 	TR - Task Assignee removed.
// 	TF - Task Followers updated.
// 	TS - Task Status updated.
// 	TP - Task priority updated.
// 	TD - Task Date updated.
//	TL - Task Deleted.

    public function save($data)
	{
		$result = array();

		$closed_status_id	= $this->get_closed_status_id();

		$this->CI->load->model('entity');
		$entity_task_status = array_key_index($this->CI->entity->get_task_status(),'task_status_id');	

		$task_priorities = $this->CI->config->item('task_priority');

		if (!empty($data['items']))
		{
			foreach ($data['items'] as $rkey => $row)
			{
				$old_task_number = !empty($row['task_number'])?$row['task_number']:0;

				if (!empty($row['task_status_id']))
				{
					$row['status_date'] = '';
					$row['is_closed']	= 0;
					if ($closed_status_id == $row['task_status_id']) {
						$row['status_date']			= $this->cur_time;
						$row['is_closed']			= 1;
						$row['actual_finish_date']	= $this->cur_time;
					} /*else {
						$row['actual_finish_date']	= null;
					}*/
				}

				if (empty($row[$this->tbl_key]))
				{
					$task_number = $this->CI->tasks->get_task_number($row['project_id']);
					// map_ids (task_number , new_task_id, new_parent_id)
					$result['map_ids'][$old_task_number][] =$task_number;

					$row['parent_task_id'] = ($row['parent_task_id'] && isset($result['map_ids'][$row['parent_task_id']][1]))?$result['map_ids'][$row['parent_task_id']][1]:$row['parent_task_id'];
					$row['task_number'] = $task_number;
					
					// reset order id
					if (empty($row['order_id'])) {
						$this->CI->db->select('MAX(order_id)+1 AS next_order_id');
						$this->CI->db->from('project_task');
						$this->CI->db->where(array('project_id' => $row['project_id'], 'parent_task_id' => $row['parent_task_id']));
						
						$query = $this->CI->db->get();
						$tmp_row = $query->row();
						
						$row['order_id'] = ($tmp_row->next_order_id)?$tmp_row->next_order_id:$row['order_id'];
					}
				}
				$pre_data = array();
				if (!empty($row[$this->tbl_key]))
				{
					$pre_data['task']		= $this->CI->tasks->get_tasks('', array('filters' => array('ts.project_id' => $data['project_id'], 'ts.task_id' => $row[$this->tbl_key])));
					$pre_data['followers'] 	= $this->CI->tasks->get_followers($data['project_id'], array('pt.task_id' => $row[$this->tbl_key]));
				}
				$pre_data['user_id'] 	= $this->user_id;
				$pre_data['entity_id']	= $this->entity_id;

				if ($task_id = parent::save($row))
				{
					$parent_task_id	= !empty($row['parent_task_id']) ? $row['parent_task_id'] : (!empty($pre_data['task']) ? $pre_data['task'][0]['parent_task_id'] : 0);
					if (!empty($parent_task_id) && !empty($row['task_status_id']))
					{
						// Recursivly update parent tasks status.
						$this->setTaskStatus($data['project_id'], $parent_task_id, $row['task_status_id']);
					}

					// map task ids with posted task ids
					$followers = array();
					if (empty($row[$this->tbl_key]))
					{
						$result['map_ids'][$old_task_number][] = $task_id;
						$result['map_ids'][$old_task_number][] = $row['parent_task_id'];
						//task project detail for hooks
						$pre_data['task_id'] = $task_id;
						$pre_data['project_id'] = $row['project_id'];
						$msg = ' Created a new task with status '.$entity_task_status[$row['task_status_id']]['name'];
						activity_log('TC',$task_id, $msg);
					}
					else if (!empty($row['task_status_id']))
					{
//						$msg=' Assign status - '.$entity_task_status[$row['task_status_id']]['name'];
						if (!empty($pre_data['task']) && $pre_data['task'][0]['task_status_id'] != $row['task_status_id'])
						{
							$msg	= 'Status - '.$entity_task_status[$pre_data['task'][0]['task_status_id']]['name'].' to '.$entity_task_status[$row['task_status_id']]['name'];
						}
						activity_log('TS',$task_id, $msg);
					}

					if (!empty($pre_data['task']))
					{
						$msg = '';
						if (!empty($data['items'][$rkey]['start_date']) && strtotime($pre_data['task'][0]['start_date'])!=strtotime($data['items'][$rkey]['start_date']))
						{
							$msg = 'Modified - Start date '.$pre_data['task'][0]['start_date'].' to '.$data['items'][$rkey]['start_date'].' ; ';
						} 
						if (!empty($data['items'][$rkey]['finish_date'])&& strtotime($pre_data['task'][0]['finish_date'])!=strtotime($data['items'][$rkey]['finish_date']))
						{
							$msg = ($msg)?$msg:'Modified - ';
							$msg .= 'FInish date '.$pre_data['task'][0]['finish_date'].' to '.$data['items'][$rkey]['finish_date'].';';
						}

						if ($msg) activity_log('TD',$task_id, $msg);

						if (!empty($data['items'][$rkey]['task_priority']))
						{
							$msg = 'Priority - '.$task_priorities[$pre_data['task'][0]['task_priority']].' to '.$task_priorities[$data['items'][$rkey]['task_priority']];
							activity_log('TP',$task_id, $msg);
						}
					}

					$ass_users = array();
					if (isset($data['items'][$rkey]['assignee_id']))
					{
						if (!$data['items'][$rkey]['assignee_id'] && isset($pre_data['task'])  && $pre_data['task'][0]['assignee_id'])
						{
							$msg = 'Assignee removed - ';
							$this->CI->db->select('concat(first_name," " , last_name) AS user_name', false);
							$this->CI->db->from('entity_users');
							$this->CI->db->where(array('entity_id' => $this->entity_id));
							$this->CI->db->where('user_id', $pre_data['task'][0]['assignee_id']);
							$ass_names = $this->CI->db->get()->result_array();
							foreach ($ass_names as $ass_val)
							{
								$msg.= ' '.$ass_val['user_name'].',';
							}
							activity_log('TR',$task_id, trim($msg,','));
						}
						else if ($data['items'][$rkey]['assignee_id'])
						{
							$msg = 'Task assigned to ';
							$this->CI->db->select('concat(first_name," " , last_name) AS user_name', false);
							$this->CI->db->from('entity_users');
							$this->CI->db->where(array('entity_id' => $this->entity_id));
							$this->CI->db->where('user_id', $data['items'][$rkey]['assignee_id']);
							$ass_names = $this->CI->db->get()->result_array();
							foreach ($ass_names as $ass_val)
							{
								$msg.= ' '.$ass_val['user_name'].',';
							}
							activity_log('TA',$task_id, trim($msg,','));
						}
					}

					if (isset($data['items'][$rkey]['followers']))
					{
						if (!empty($data['items'][$rkey]['followers']))
						{
							foreach($data['items'][$rkey]['followers'] as $aval)
							{
								$followers[] = array('entity_id'=>$this->entity_id,'task_id' => $task_id, 'user_id' => $aval);
								$ass_users[] = $aval;
							}

							$this->CI->db->where(array('entity_id'=> $this->entity_id, 'task_id' => $task_id));
							$this->CI->db->delete('project_task_users');

							$this->CI->db->insert_batch('project_task_users', $followers);
						}

						$msg = 'Followers removed ';							
						if ($ass_users)
						{
							$msg = '';
							if (!empty($row[$this->tbl_key]) || count($ass_users)>1 || !in_array($this->user_id,$ass_users))
							{
								$msg = 'Followers - ';
								$this->CI->db->select('concat(first_name," " , last_name) AS user_name', false);
								$this->CI->db->from('entity_users');
								$this->CI->db->where(array('entity_id' => $this->entity_id));
								$this->CI->db->where_in('user_id', $ass_users);
								$ass_names = $this->CI->db->get()->result_array();
								foreach ($ass_names as $ass_val)
								{
									$msg.= ' '.$ass_val['user_name'].',';
								}
							}
						}
						if ($msg)
							activity_log('TF',$task_id, trim($msg,','));
					}

					$pre_data['task_url'] = site_url('tasks');
					run_background_process("newTaskEvent", json_encode($pre_data));
						
					$result['task_ids'][] = $task_id;
				}
				else
				{
					break;	
				}				
			}

			// get worked hours for all task
			$this->CI->db->select_sum('hours');
			$this->CI->db->select('project_task_id AS task_id');
			$this->CI->db->from('time_sheet');
			$this->CI->db->where(array('entity_id'=>$this->entity_id));
			$this->CI->db->where_in('project_task_id',$result['task_ids']);
			$this->CI->db->group_by('project_task_id');
			$query = $this->CI->db->get();
			$tmp = $query->result_array();

			$sum_hours = array();
			if ($tmp)
			{
				foreach ($tmp as $tval)
				{
					$sum_hours[$tval['task_id']] = $tval['hours'];
				}
			}

			foreach ($data['items'] as $rkey => $row)
			{
				$update = array();
				// predessors calculation
				if (!empty($row['predecessors']))
				{
					$split = explode(',',$row['predecessors']);
					$predecessors = array();
					foreach ($split as $pval)
					{
						preg_match("/(\d+)(.*)/",$pval, $match);
						if (isset($result['map_ids'][$match[1]]))
							$pval = preg_replace("/(\d+)(.*)/",$result['map_ids'][$match[1]][0].'${2}',$pval);
						$predecessors[] = $pval;
					}
					$result['predecessors'][$result['task_ids'][$rkey]] = $update['predecessors'] = implode(',',$predecessors);
				}
				
				// update worked hours
				$update['worked_hours'] = !empty($sum_hours[$result['task_ids'][$rkey]])?$sum_hours[$result['task_ids'][$rkey]]:0;
				 
				$this->CI->db->where('task_id' , $result['task_ids'][$rkey]);				
				$this->CI->db->update('project_task', $update);
			}
		}

		if (!empty($data['deletedItems']) && $deleted_ids = $this->get_deleted_ids($data['deletedItems'],array(),true))
		{
			$this->CI->db->where_in('task_id' , $deleted_ids);
			$this->CI->db->delete('project_task');
			
			if ($deleted_ids) {
				$result['deleted_ids'] = $deleted_ids;
			}
		}

		return $result;
	}

	public function get_deleted_ids ($deleted_ids, $return = array(),$root=false)
	{

		foreach($deleted_ids as $key => $id) {
			unset($deleted_ids[$key]);
			if ($root && is_member()) {
				$this->CI->db->select('task_id');
				$this->CI->db->from('project_task');
				$this->CI->db->where(array('entity_id' => $this->entity_id, 'created_by'=>$this->user_id, 'task_id'=>$id));
			
				$query = $this->CI->db->get();
				if (!$row = $query->row()) {
					continue;
				}
			}
			$this->CI->db->select('task_id');
			$this->CI->db->from('project_task');
			$this->CI->db->where(array('entity_id' => $this->entity_id, 'parent_task_id'=>$id));
			
			$query = $this->CI->db->get();
			$result = $query->result_array();
			
			$data = array();
			if ($result) {
				foreach ($result as $val)
				{
					$data[] = $val['task_id'];
				}
				
				$chi_return = $this->get_deleted_ids($data, $return);
				$return = ($chi_return && $return)?array_merge($chi_return,$return):(($chi_return)?$chi_return:$return);
			}
			activity_log('TD',$id, $this->CI->ot_user->get('first_name').' '.$this->CI->ot_user->get('last_name').'  deleted task ');
			$return[] = $id;
		}
		
		return $return;
	}
    
    
 
	public function delete($client_id)
	{
		$this->CI->db->select('address_id');
		$this->CI->db->from('client_contact');
		$this->CI->db->where('client_id',$client_id);
		$query = $this->CI->db->get();
		$addr_ids = array();
		
		// delete address detail	
		if ($query->num_rows() > 0)
		{
		   foreach ($query->result() as $row)
		   {
		      $addr_ids[] =  $row->address_id;
		   }
		   $this->CI->db->where('entity_id', $this->ot_app->get('entity_id'));
		   $this->CI->db->where_in('address_id', $addr_ids);
		   $this->CI->db->delete('address');
		} 
		// delete client contanct
		$this->CI->db->where(array('entity_id' => $this->ot_app->get('entity_id'),'client_id'=>$client_id));		
		$this->CI->db->delete('client_contact');
		
		//delete client
		$this->CI->db->where(array('entity_id' => $this->ot_app->get('entity_id'),'client_id'=>$client_id));
		$this->CI->db->delete('client');
		
		return true;
	}
	
}
