<?php if ( ! defined('BASEPATH')) exit('No direct script access allowed');

require_once(APPPATH . '/models/project/mod.project_member.php');

class Project_pm extends Project_member
{
	function __construct()
	{
		parent::__construct();
	}

	public function get_project($project_id)
	{
		$this->db->select('pr.project_id, cl.client_id, concat(cl.client_code,cl.client_name) as client_name, pr.project_name, pr.description, pr.project_code, pr.members_count', false);
		$this->db->from('project as pr');
		$this->db->join('project_users as pru', 'pr.entity_id=pru.entity_id and pr.project_id=pru.project_id', 'INNER');
		$this->db->join('client as cl', 'pr.entity_id = cl.entity_id and pr.client_id = cl.client_id', 'INNER');
		$this->db->where(array('pr.entity_id' => $this->ot_app->get('entity_id'), 'pru.user_id'=>$this->ot_user->get('user_id'), 'pr.project_id' => $project_id));
		$query = $this->db->get();

		return $query->result();
	}
	
	public function get_project_members($project_id)
	{
		$this->db->select('pru.project_user_id, pru.user_role, pru.estimated_hours, u.user_id, u.first_name, u.last_name, u.user_image,pru.actual_hours', false);
		$this->db->from('project_users as pru');
		$this->db->join('entity_users as u', 'pru.entity_id=u.entity_id and pru.user_id=u.user_id', 'INNER');
		//$this->db->join('time_sheet AS ts', 'pru.entity_id = ts.entity_id and pru.user_id = ts.user_id AND pru.project_id = ts.project_id', 'left');
		$this->db->where(array('pru.entity_id' => $this->ot_app->get('entity_id'), 'pru.project_id' => $project_id));
		$this->db->group_by('pru.project_user_id');
		$this->db->order_by('u.first_name, u.last_name');
		
		$query = $this->db->get();		
		return $query->result_array();
	}

	public function get_user_roles()
	{
		$return	= $this->config->config['user_role'];
		unset($return['A']);
		unset($return['F']);
		unset($return['P']);

		return $return;
	}

	public function get_project_attachments($project_id)
	{
		$this->db->select('a.file_type, a.file_name, a.file_path, a.file_size, a.created_by, a.created_date');
		$this->db->from('project as pr');
		$this->db->join('project_users as pru', 'pr.entity_id=pru.entity_id and pr.project_id=pru.project_id', 'INNER');
		$this->db->join('attachment as a', 'pr.entity_id=a.entity_id and pr.project_id=a.module_id and a.module_type="P"', 'INNER');
		$this->db->where(array('pr.entity_id' => $this->ot_app->get('entity_id'), 'pru.user_id'=>$this->ot_user->get('user_id'), 'pr.project_id' => $project_id));
		$this->db->order_by('a.created_by', 'desc');

		$query = $this->db->get();
		return $query->result_array();
	}

	public function get_invoiced_hours ($project_id)
	{
		$this->db->select('sum(tm.hours) as invoiced_hours');
		$this->db->from('project_task as prt');
		$this->db->join('time_sheet as tm', 'prt.entity_id=tm.entity_id and prt.project_id=tm.project_id', 'INNER');
		$this->db->join('project_users as pru', 'prt.entity_id=pru.entity_id and prt.project_id=pru.project_id', 'INNER');
		$this->db->where(array('prt.entity_id' => $this->ot_app->get('entity_id'), 'pru.user_id'=>$this->ot_user->get('user_id'), 'prt.project_id' => $project_id, 'tm.is_billable'=>0, 'tm.invoice_id >' => 0));

		$query = $this->db->get();
		$result = $query->row();

		return ($result->invoiced_hours)?$result->invoiced_hours:0;
	}

	public function store_project_task_types($project_id, $data)
	{
		$this->load->library('project_task_types_table');

		//remove deleted tasks
		if (!empty($data['deleted_items']) && !$this->project_task_types_table->delete($project_id, $data['deleted_items']))
		{
			return false;
		}

		$result = array();$id = true;
		$this->load->library('table_fields');
			
		if (!empty($data['task_types']))
		{
			foreach ($data['task_types'] as $row)
			{

				$row['project_id'] = $project_id;
				$row = $this->table_fields->strip_fields('tasktypes', $row);
					
				if ($id = $this->project_task_types_table->save($row))
				{
					$result[] = $id;
				}
			}
		}
		// update budget value
		$this->load->library('project_table');
		$this->project_table->update_budget_value(array('project_id'=>$project_id));
		
		$this->response->set_data('project_task_type_id', $result);
		return $id;
	}

	public function store_project_members($project_id, $data)
	{
		$this->load->library('project_users_table');

		//remove deleted tasks
		if (!empty($data['deleted_items']) && !$this->project_users_table->delete($project_id, $data['deleted_items']))
		{
			return false;
		}

		if (!empty($data['members']))
		{
			foreach ($data['members'] as $row)
			{
				// prepare required detail
				/*if (empty($row[$this->project_users_table->tbl_key]))
				$row['user_role'] = 'M';*/

				$row['project_id'] = $project_id;
				$id = $this->project_users_table->save($row);
				if (!$id)
				{
					return false;
				}
			}
		}
		
		$this->reset_members_count($project_id);
		// update budget value
		$this->load->library('project_table');
		$this->project_table->update_budget_value(array('project_id'=>$project_id));
		
		return $id;
	}
	
	public function get_project_tasks($project_id)
	{
		$cur_date = $this->auth->get_timezone_date('Y-m-d');
		$this->db->select('sum(CASE WHEN ts.is_closed=0 AND (finish_date<"'.$cur_date.'" OR finish_date<=created_date) THEN 1 ELSE 0 END) AS over_due_count', false);
		$this->db->select('sum(CASE WHEN ts.is_closed=0 THEN 1 ELSE 0 END) AS open_task_count', false);
		$this->db->select('sum(CASE WHEN ts.is_closed=1 THEN 1 ELSE 0 END) AS close_task_count', false);
		$this->db->from('project_task AS pt');
		$this->db->join('task_status AS ts', 'pt.entity_id=ts.entity_id and pt.task_status_id=ts.task_status_id', 'INNER');
		$this->db->where(array('pt.entity_id'=>$this->ot_app->get('entity_id'), 'pt.project_id' => $project_id));
		$query = $this->db->get();
// echo $this->db->last_query();
		return $query->row_array();
	}
	
	public function archive($project_id, $data=array())
	{
		if ($data['is_active'] && !license_limit($project_id, 'projects')) {
			$this->response->error(sprintf(lang('UPGRADE_YOUR_VERSION'),'projects'));
			return false;
		}

		$this->db->where(array('project_id'=>$project_id, 'entity_id'=>$this->ot_app->get('entity_id')));
		$this->db->update('project',array('is_active'=>$data['is_active']));
		return true;
	}
	
	public function delete($project_id)
	{
		$this->load->library('project_table');
		$this->project_table->delete($project_id,array('created_by' =>$this->ot_user->get('user_id')));
		return true;
	}
	
	public function store_project_document($project_id, $data)
	{
		$this->load->library('attachment_table');
		
		if(!empty($data['new_attachment'])) 
		{
			foreach($data['new_attachment'] as $key=>$value) 
			{
				$attachment_id = $this->attachment_table->save($value);
				if($value['attachment_id']==0 && $attachment_id) 
				{ 
					return $attachment_id;
				}
			}
		}
		return true;
	}
}
